/** emacs: this is -*- c++ -*- **/
/**
 **   @file    histogram.h        
 **                   
 **   @author  sutt
 **   @date    Mon 28 Dec 2020 19:33:12 GMT
 **
 **   $Id: histogram.h, v0.0   Mon 28 Dec 2020 19:33:12 GMT sutt $
 **
 **   Copyright (C) 2020 sutt (sutt@cern.ch)    
 **
 **/


#include "appl_grid/histogram.h"

#include "appl_grid/serialise_base.h"



histogram::histogram( const std::string& name ) : mname(name) { } 


histogram::histogram( const std::string& name, size_t nbins, const double* limits ) : mname(name) {
  if ( nbins>0 ) create( nbins, limits );
  else throw exception("histogram: not enough bins creating histogram: "+name);
}


histogram::histogram( const std::string& name, const std::vector<double>& limits ) : mname(name) {
    if ( limits.size()>1 ) create( limits.size()-1, &limits[0] );
    else throw exception("histogram: not enough bin limits creating histogram: "+name);
}


histogram::histogram( const std::string& name, size_t nbins, double lo, double hi ) : mname(name) {    
    if ( nbins==0 ) throw exception("histogram: not enough bins creating histogram: "+name);
    std::vector<double> limits(nbins+1,0);
    for ( size_t i=nbins ; i-- ; ) limits[i] = (lo*(nbins-i) + hi*i)/nbins;
    /// make sure the limits at least are exact 
    limits[0]     = lo;
    limits[nbins] = hi;
    create( nbins, &limits[0] );
} 


/// constructor from stream

histogram::histogram( const std::vector<SB::TYPE> stream ) { 
    deserialise( stream );
}


histogram::histogram( const histogram& h ) : 
    mname(h.mname),
    mxlimits(h.mxlimits),
    mx(h.mx),
    my(h.my),
    mye(h.mye) 
{ }


histogram& histogram::operator=( histogram&& h ) {
    std::swap( mname, h.mname );
  
    std::swap( mxlimits, h.mxlimits );
    std::swap( mx, h.mx );

    std::swap( my,  h.my );
    std::swap( mye, h.mye );
    
    return *this;
}


histogram& histogram::operator=( const histogram& h ) {
    mname    = h.mname;
    mxlimits = h.mxlimits;
    mx       = h.mx;
    my       = h.my;
    mye      = h.mye;
    return *this;
}


void histogram::fill( double x, double w ) {
    int ibin = index( x );
    if ( ibin != -1 ) { 
      my[ibin] += w;
      mye[ibin] = std::sqrt( mye[ibin]*mye[ibin] + w );
    } 
}
  

  /// serialisation and deserialisation ...

std::vector<SB::TYPE> histogram::serialise() const { 
   
  std::vector<SB::TYPE> s;
    
    s.push_back( SB::WRITEGUARD );

    s.push_back(0); // overall size

    SB::serialise( s, name() );
    SB::serialise( s, mxlimits );
    /// don't need to save this - can compute from the limits
    /// SB::serialise( s, mx );
    SB::serialise( s, my );
    SB::serialise( s, mye );

    s.push_back( SB::WRITEGUARD );

    s[1] = s.size();

    return s;
}


void histogram::deserialise( const std::vector<SB::TYPE>& v ) { 
 
  std::vector<SB::TYPE>::const_iterator itr = v.begin();
    
    if ( (*itr++) != SB::WRITEGUARD ) throw exception("histogram: read error");
   
    /// don't need this now ...
    //  int ntotal = (*itr++); 
    (*itr++);

    SB::deserialise( itr, mname );
    SB::deserialise( itr, mxlimits );
    /// SB::deserialise( itr, mx );
    SB::deserialise( itr, my );
    SB::deserialise( itr, mye );
    
    SB::TYPE var = (*itr++);

    //    std::cout << "read: " << mname << "\tsize: " << ntotal << std::endl;
    
    if ( var != SB::WRITEGUARD ) throw exception("histogram: read error reading histogram: "+name());

    mx.resize(my.size());
    for ( size_t i=my.size() ; i-- ; ) mx[i] = 0.5*(mxlimits[i]+mxlimits[i+1]);
   
} 


void histogram::create( size_t nbins, const double* limits ) { 
    
    mxlimits.resize(nbins+1);
    
    for ( size_t i=nbins+1 ; i-- ; ) mxlimits[i] = limits[i];
    
    mx.resize(nbins);
    
    for ( size_t i=nbins ; i-- ; ) mx[i] = 0.5*(limits[i]+limits[i+1]);
    
    my  = std::vector<double>(nbins,0);
    mye = std::vector<double>(nbins,0);
   
}











